/*
 * mp2.c:	Use the S/P-DIF interface for redirecting PCM stream
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 *
 * Copyright (C) 2003 Werner Fink, <werner@suse.de>
 */

#include "types.h"
#include "mp2.h"
#include "mp2_decoder.h"

static MP2Decoder MP2Dec;

static frame_t & scan_mp2_frame(const uint_8 *&out, const uint_8 *const tail);
iec60958 mp2(scan_mp2_frame, 48000, SPDIF_BURST_SIZE, 0);

// This function requires two arguments:
//   first is the start of the data segment
//   second is the tail of the data segment
static frame_t & scan_mp2_frame(const uint_8 *&out, const uint_8 *const tail)
{
    static size_t pos = 0;
    // We overwrite the none audio magic PCM header (offset is zero)
    const static size_t payload_size = SPDIF_BURST_SIZE;

    debug("<<<<<<<< MP2_SCAN size 0x%X, pos=0x%X, left=0x%X\n", 
		tail-out, pos, payload_size-pos);

    mp2.play.burst = (uint_32 *)0;
    mp2.play.size = 0;

    /**
     * now we do decode as much as we can .. 
     */
    if (tail > out ) {
	debug("MP2_SCAN push pos=0x%X, left=0x%X, size=0x%X\n", 
		pos, payload_size-pos, tail-out);
	MP2Dec.pushMP2Data ( out, tail-out );
	out += tail-out ;
    }

    if ( MP2Dec.remaining_mp2 == 0 && pos<payload_size ) {
	debug("MP2_SCAN NO DATA LEFT to fill (1) pos=0x%X, left=0x%X, rem_mp2=0x%X\n", 
		pos, payload_size-pos, MP2Dec.remaining_mp2);
	goto done;
    }

    debug("MP2_SCAN 1st decode pos=0x%X, left=0x%X, rem_mp2=0x%X\n", 
		pos, payload_size-pos, MP2Dec.remaining_mp2);

    MP2Dec.setExternalPCMData( &mp2.current[pos], payload_size-pos );
    while(pos < payload_size && MP2Dec.decode()) {
	    pos += MP2Dec.pcm_netto_size;
	    debug("MP2_SCAN decoded done_mp2=0x%X rem_mp2=0x%X, rem_samples=0x%X, pcm=0x%X, pos=0x%X, left=0x%X\n", 
			MP2Dec.digested_mp2, MP2Dec.remaining_mp2, 
			MP2Dec.remaining_samples, MP2Dec.pcm_netto_size,
			pos, payload_size-pos);
	    MP2Dec.setExternalPCMData( &mp2.current[pos], payload_size-pos );
    }
    if ( pos < payload_size ) {
	debug("MP2_SCAN NO DATA LEFT to fill (2) pos=0x%X, left=0x%X, rem_mp2=0x%X\n", 
		pos, payload_size-pos, MP2Dec.remaining_mp2);
	goto done;
    }

    swab(mp2.payload, mp2.payload, pos);

    mp2.play.burst = (uint_32 *)(&mp2.current[0]);
    mp2.play.size = payload_size;
    mp2.play.pay = pos;

    mp2.Switch();

    mp2.last.burst = (uint_32 *)(&mp2.remember[0]);
    mp2.last.size = mp2.play.size;
    mp2.last.pay = mp2.play.pay;

    debug(">>>>>>>> %d, pos=%d\n", tail-out, pos);

    pos = 0;
done:
    return mp2.play;
}
